package com.sztzjy.money_management.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@Api(tags = "产品中心相关")
@RequestMapping("api/ProductCenter")
public class ProductCenter {
    @Autowired
    FundProductMapper fundProductMapper;
    @Autowired
    RealGoldProductMapper realGoldProductMapper;
    @Autowired
    BankDepositsMapper bankDepositsMapper;
    @Autowired
    InsuranceProMapper insuranceProMapper;
    @Autowired
    p2pProductMapper p2pProductMapper;
    @Autowired
    ObjectiveQuestionMapper objectiveQuestionMapper;

    @PostMapping("selectFundProduct")
    @ApiOperation("基金产品查询")
    @AnonymousAccess
    public ResultEntity<PageInfo<FundProduct>> selectFundProduct(@RequestParam(required = false) String param,
                                                                 @RequestParam Integer index,
                                                                 @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        FundProductExample example = new FundProductExample();
        if (param != null) {
            FundProductExample.Criteria criteria1 = example.createCriteria();
            FundProductExample.Criteria criteria2 = example.createCriteria();
            FundProductExample.Criteria criteria3 = example.createCriteria();
            criteria1.andFundCodeLike("%" + param + "%");
            criteria2.andFundNameLike("%" + param + "%");
            criteria3.andFundTypeLike("%" + param + "%");
            example.or(criteria1);
            example.or(criteria2);
            example.or(criteria3);
        }
        example.setOrderByClause("update_date asc");
        List<FundProduct> fundProducts = fundProductMapper.selectByExample(example);
        PageInfo<FundProduct> pageInfo = new PageInfo<>(fundProducts);
        return new ResultEntity<>(HttpStatus.OK, "基金产品查询成功！", pageInfo);

    }

    @PostMapping("selectBankDeposits")
    @ApiOperation("银行储蓄查询")
    @AnonymousAccess
    public ResultEntity<PageInfo<BankDeposits>> selectBankDeposits(@RequestParam(required = false) String bankName,
                                                                   @RequestParam String schoolId,
                                                                   @RequestParam Integer index,
                                                                   @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        BankDepositsExample example = new BankDepositsExample();
        BankDepositsExample.Criteria criteria = example.createCriteria();
        if (bankName != null) {
            criteria.andBankNameLike("%" + bankName + "%");
        }
        List<BankDeposits> bankDepositsList = bankDepositsMapper.selectByExample(example);
        PageInfo<BankDeposits> pageInfo = new PageInfo<>(bankDepositsList);
        return new ResultEntity<>(HttpStatus.OK, "银行储蓄查询成功！", pageInfo);
    }

    @PostMapping("updateBankDeposits")
    @ApiOperation("银行储蓄编辑")
    @AnonymousAccess
    public ResultEntity updateBankDeposits(@RequestBody BankDeposits bankDeposits) {
        bankDepositsMapper.updateByPrimaryKeySelective(bankDeposits);
        return new ResultEntity<>(HttpStatus.OK, "银行储蓄编辑成功");
    }


    @PostMapping("selectP2PProduct")
    @ApiOperation("P2P产品查询")
    @AnonymousAccess
    public ResultEntity<PageInfo<p2pProduct>> selectP2PProduct(@RequestParam(required = false) String productName,
                                                               @RequestParam Integer index,
                                                               @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        p2pProductExample example = new p2pProductExample();
        if (productName != null) {
            p2pProductExample.Criteria criteria = example.createCriteria();
            criteria.andP2pNameLike("%" + productName + "%");
        }
        example.setOrderByClause("update_time asc");
        List<p2pProduct> p2pProducts = p2pProductMapper.selectByExample(example);
        PageInfo<p2pProduct> pageInfo = new PageInfo<>(p2pProducts);
        return new ResultEntity<>(HttpStatus.OK, "P2P产品查询成功！", pageInfo);
    }


    @PostMapping("selectRealGoldProduct")
    @ApiOperation("实务黄金查询")
    @AnonymousAccess
    public ResultEntity<PageInfo<RealGoldProduct>> selectRealGoldProduct(@RequestParam Integer index,
                                                                         @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        RealGoldProductExample example = new RealGoldProductExample();
        example.setOrderByClause("trading_day asc");
        List<RealGoldProduct> realGoldProducts = realGoldProductMapper.selectByExample(example);
        PageInfo<RealGoldProduct> pageInfo = new PageInfo<>(realGoldProducts);
        return new ResultEntity<>(HttpStatus.OK, "实务黄金查询成功！", pageInfo);
    }

    @PostMapping("selectInsurancePro")
    @ApiOperation("保险产品查询")
    @AnonymousAccess
    public ResultEntity<PageInfo<InsuranceProWithBLOBs>> selectInsurancePro(@ApiParam("保险子类") @RequestParam(required = false) String kind,
                                                               @ApiParam("搜索参数") @RequestParam(required = false) String name,
                                                               @RequestParam Integer index,
                                                               @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        InsuranceProExample example = new InsuranceProExample();
        InsuranceProExample.Criteria criteria = example.createCriteria();
        if (kind != null) {
            criteria.andKindEqualTo(kind);
        }
        if (name != null) {
            criteria.andNameLike("%" + name + "%");
        }
        List<InsuranceProWithBLOBs> insurancePros = insuranceProMapper.selectByExampleWithBLOBs(example);
        PageInfo<InsuranceProWithBLOBs> pageInfo = new PageInfo<>(insurancePros);
        return new ResultEntity<>(HttpStatus.OK, "保险产品查询成功！", pageInfo);
    }


}
