package com.sztzjy.money_management.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.config.Constant;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.SynthesisPlanScoreDto;
import com.sztzjy.money_management.entity.dto.UserStatisticsDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.SynthesisPlanService;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Pattern;

//综合规划相关
@RestController
@Api(tags = "综合规划相关")
@RequestMapping("api/SynthesisPlanController")
public class SynthesisPlanController {
    @Autowired
    SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    WorkScheduleMapper workScheduleMapper;
    @Autowired
    SynthesisPlanScoreMapper synthesisPlanScoreMapper;
    @Autowired
    UserInfoMapper userInfoMapper;
    @Autowired
    CaseInfoMapper caseInfoMapper;
    @Autowired
    UserStatisticsMapper userStatisticsMapper;
    @Autowired
    CaseFinanceLiabilityMapper caseFinanceLiabilityMapper;
    @Autowired
    ClientFamilyInfoMapper clientFamilyInfoMapper;
    @Autowired
    SynthesisPlanService synthesisPlanService;

    //目标客户查询
    //根据create_time排序
    @AnonymousAccess
    @ApiOperation("目标客户查询")
    @PostMapping("selectTargetCustomer")
    public ResultEntity<PageInfo<SynthesisPlanClient>> selectTargetCustomer(@RequestParam String userId,
                                                                            @RequestParam String schoolId,
                                                                            @RequestParam Integer index,
                                                                            @RequestParam Integer size,
                                                                            @ApiParam("客户姓名/身份证号") @RequestParam(required = false) String param) {
        //先根据userID和source查询所有数据 注意 学生跟踪客户后 会生成一条SynthesisPlanClient数据，需要去重
        PageHelper.startPage(index, size);
        List<String> sourceList = new ArrayList<>();
        sourceList.add(Constant.BUILT_IN_SCHOOL_ID);
        sourceList.add(schoolId);
        SynthesisPlanClientExample neiZhiExample = new SynthesisPlanClientExample();
        SynthesisPlanClientExample.Criteria criteria = neiZhiExample.createCriteria();
        if (StringUtils.isNotBlank(param)) {
            if (Pattern.matches("\\d+", param)) {
                criteria.andIdcardLike("%" + param + "%");
            } else {
                criteria.andNameLike("%" + param + "%");
            }
        }
        criteria.andUseridIsNull().andSourceIn(sourceList);
        neiZhiExample.setOrderByClause("create_time asc");
        List<SynthesisPlanClient> synthesisPlanClients = synthesisPlanClientMapper.selectByExample(neiZhiExample);

        //获取领取状态
        for (int i = 0; i < synthesisPlanClients.size(); i++) {
            SynthesisPlanClientExample example = new SynthesisPlanClientExample();
            example.createCriteria().andUseridEqualTo(userId).andCaseidEqualTo(synthesisPlanClients.get(i).getCaseid()).andGetStatusEqualTo("true");
            List<SynthesisPlanClient> stuClients = synthesisPlanClientMapper.selectByExample(example);
            if (!stuClients.isEmpty()) {
                synthesisPlanClients.get(i).setGetStatus("true");
            }
        }
        PageInfo<SynthesisPlanClient> pageInfo = new PageInfo<>(synthesisPlanClients);
        return new ResultEntity<>(pageInfo);
    }

    //目标客户跟踪
    @AnonymousAccess
    @ApiOperation("目标客户跟踪")
    @PostMapping("TrackDownTargetCustomer")
    public Boolean TrackDownTargetCustomer(@RequestParam String userId, @RequestParam String stuClientId, @RequestParam String schoolId) {
        SynthesisPlanClient synthesisPlanClient = synthesisPlanClientMapper.selectByPrimaryKey(stuClientId);
        synthesisPlanClient.setStuClientId(String.valueOf(UUID.randomUUID()));
        synthesisPlanClient.setGetStatus("true");
        synthesisPlanClient.setGetTime(new Date());
        synthesisPlanClient.setUserid(userId);
        synthesisPlanClient.setSchoolId(schoolId);
        int i = synthesisPlanClientMapper.insert(synthesisPlanClient);

        //判断用户领取的时候判断资产负载表 总净值大于600W 潜在客户+1
        CaseFinanceLiabilityExample example = new CaseFinanceLiabilityExample();
        example.createCriteria().andUserIdEqualTo(userId).andCaseIdEqualTo(synthesisPlanClient.getCaseid());
        List<CaseFinanceLiability> caseFinanceLiabilitieList = caseFinanceLiabilityMapper.selectByExample(example);
        if (caseFinanceLiabilitieList.isEmpty()) {
            return i == 1;
        }
        CaseFinanceLiability caseFinanceLiability = caseFinanceLiabilitieList.get(0);
        BigDecimal totalNet = caseFinanceLiability.getTotalNet() == null ? BigDecimal.ZERO : caseFinanceLiability.getTotalNet();
        if (totalNet.compareTo(BigDecimal.valueOf(6000000)) >= 0) {  //高净值用户
            //先判断 是否创建用户统计对象
            UserStatistics userStatistics = userStatisticsMapper.selectByPrimaryKey(userId);
            if (userStatistics == null) {
                UserStatistics createUserStatistics = new UserStatistics();
                createUserStatistics.setUserid(userId);
                createUserStatistics.setHighNetClientHaveCount(0);
                createUserStatistics.setHighNetClientLatentCount(1);
                userStatisticsMapper.insert(createUserStatistics);
            } else {  //存在这个统计对象
                userStatistics.setHighNetClientLatentCount(userStatistics.getHighNetClientLatentCount() + 1);
                userStatisticsMapper.updateByPrimaryKey(userStatistics);
            }

        }
        return i == 1;
    }

    //工作日程新增与编辑
    @AnonymousAccess
    @ApiOperation("工作日程新增与编辑")
    @PostMapping("insertAndUpdateWorkSchedule")
    public Boolean insertWorkSchedule(@RequestParam String userId, @RequestBody WorkSchedule workSchedule) {
        if (StringUtils.isBlank(workSchedule.getWorkScheduleId())) {  //代表新增
            workSchedule.setWorkScheduleId(String.valueOf(UUID.randomUUID()));
            workSchedule.setUserId(userId);
            int i = workScheduleMapper.insert(workSchedule);
            return i == 1;
        } else { //编辑
            int i = workScheduleMapper.updateByPrimaryKeySelective(workSchedule);
            return i == 1;
        }
    }

    //工作日程列表查询
    @AnonymousAccess
    @ApiOperation("工作日程列表查询")
    @PostMapping("selectWorkScheduleList")
    public ResultEntity<PageInfo<WorkSchedule>> selectWorkScheduleList(@RequestParam String userId,
                                                                       @ApiParam("开始时间") @RequestParam Date startTime,
                                                                       @ApiParam("结束时间") @RequestParam Date endTime, @RequestParam Integer index, @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        WorkScheduleExample example = new WorkScheduleExample();
        WorkScheduleExample.Criteria criteria = example.createCriteria();
        criteria.andUserIdEqualTo(userId).andOrderTimeBetween(startTime, endTime);
        List<WorkSchedule> workSchedules = workScheduleMapper.selectByExample(example);
        PageInfo<WorkSchedule> pageInfo = new PageInfo<>(workSchedules);
        return new ResultEntity<>(HttpStatus.OK, "工作日程列表查询成功！", pageInfo);
    }

    //工作日程详情查询
    @AnonymousAccess
    @ApiOperation("工作日程详情查询")
    @GetMapping("selectWorkSchedule")
    public ResultEntity<WorkSchedule> selectWorkSchedule(@RequestParam String workScheduleId) {
        WorkSchedule workSchedule = workScheduleMapper.selectByPrimaryKey(workScheduleId);
        return new ResultEntity<>(HttpStatus.OK, "工作日程详情查询成功！", workSchedule);
    }

    //工作日程删除
    @AnonymousAccess
    @ApiOperation("工作日程删除")
    @GetMapping("deleteWorkSchedule")
    public ResultEntity deleteWorkSchedule(@RequestParam String workScheduleId) {
        int i = workScheduleMapper.deleteByPrimaryKey(workScheduleId);
        if (i == 1) {
            return new ResultEntity<>(HttpStatus.OK, "工作日程删除成功！");
        }
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "工作日程删除失败！");
    }


    //规划报告列表展示
    @AnonymousAccess
    @ApiOperation("规划报告列表展示")
    @PostMapping("selectPlanningReportList")
    public ResultEntity<PageInfo<SynthesisPlanClient>> selectPlanningReportList(@RequestParam String userId,
                                                                                @ApiParam("理财类型") @RequestParam String manageMoneyMattersType,
                                                                                @ApiParam("提交状态") @RequestParam String submitStatus,
                                                                                @ApiParam("查询参数") @RequestParam String param, @RequestParam Integer index, @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        SynthesisPlanClientExample example = new SynthesisPlanClientExample();
        SynthesisPlanClientExample.Criteria criteria = example.createCriteria();
        criteria.andUseridEqualTo(userId);
        if (StringUtils.isBlank(submitStatus)) {
            List<String> submitStatusList = new ArrayList<>();
            submitStatusList.add("未审核");
            submitStatusList.add("已审核");
            submitStatusList.add("未提交");
            criteria.andSubmitStatusIn(submitStatusList);
        } else {
            criteria.andSubmitStatusEqualTo(submitStatus);
        }
        if (StringUtils.isNotBlank(manageMoneyMattersType)) {
            criteria.andManageMoneyMattersTypeEqualTo(manageMoneyMattersType);
        }
        if (StringUtils.isNotBlank(param)) {
            SynthesisPlanClientExample.Criteria criteria1 = example.createCriteria();
            SynthesisPlanClientExample.Criteria criteria2 = example.createCriteria();
            SynthesisPlanClientExample.Criteria criteria3 = example.createCriteria();
            SynthesisPlanClientExample.Criteria criteria4 = example.createCriteria();
            criteria1.andProposalNumberLike("%" + param + "%");
            criteria2.andProposalNameLike("%" + param + "%");
            criteria3.andNameLike("%" + param + "%");
            criteria4.andIdcardLike("%" + param + "%");
            example.or(criteria1);
            example.or(criteria2);
            example.or(criteria3);
            example.or(criteria4);
        }
        example.setOrderByClause("create_time asc");
        List<SynthesisPlanClient> synthesisPlanClients = synthesisPlanClientMapper.selectByExample(example);
        PageInfo<SynthesisPlanClient> pageInfo = new PageInfo<>(synthesisPlanClients);
        return new ResultEntity<>(HttpStatus.OK, "规划报告列表展示成功！", pageInfo);
    }

    //规划报告提交
    @AnonymousAccess
    @ApiOperation("规划报告提交")
    @PostMapping("submitPlanningReport")
    @Transactional
    public ResultEntity<HttpStatus> submitPlanningReport(@RequestParam String stuClientId,
                                                         @RequestParam String userId,
                                                         @RequestParam String caseId) {
        SynthesisPlanClient synthesisPlanClient = synthesisPlanClientMapper.selectByPrimaryKey(stuClientId);
        if (synthesisPlanClient.getUserid().equals(userId) && synthesisPlanClient.getSubmitStatus().equals("未提交")) {
            //提交完成 设置用户统计 高净值已有客户总数

            //增加成绩方法 计算成绩方法
            ResultEntity<HttpStatus> score = synthesisPlanService.getScore(userId, caseId);
            if (!score.getStatusCode().equals(HttpStatus.OK)) {
                return score;
            }
            synthesisPlanClient.setSubmitStatus("未审核");
            synthesisPlanClientMapper.updateByPrimaryKeySelective(synthesisPlanClient);
            return new ResultEntity<>(HttpStatus.OK, "规划报告提交成功！");
        }
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "规划报告提交失败！提交状态错误");
    }

    //已有客户、规划报告删除
    @AnonymousAccess
    @ApiOperation("已有客户、规划报告删除")
    @PostMapping("deletePlanningReport")
    public ResultEntity deletePlanningReport(@RequestParam String stuClientId) {
        int i = synthesisPlanClientMapper.deleteByPrimaryKey(stuClientId);
        if (i == 1) {
            return new ResultEntity<>(HttpStatus.OK, "删除成功！");
        }
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败！");
    }

    //规划报告审核报告展示
    @AnonymousAccess
    @ApiOperation("规划报告审核报告展示")
    @PostMapping("selectSynthesisPlanScoreList")
    public ResultEntity<SynthesisPlanScoreDto> selectSynthesisPlanScoreList(@RequestParam String caseId, @RequestParam String userId, @RequestParam Integer index, @RequestParam Integer size) {
        //获取审核类容
        PageHelper.startPage(index, size);
        SynthesisPlanScoreExample example = new SynthesisPlanScoreExample();
        example.createCriteria().andCaseidEqualTo(caseId).andUseridEqualTo(userId);
        List<SynthesisPlanScore> synthesisPlanScores = synthesisPlanScoreMapper.selectByExample(example);
        PageInfo<SynthesisPlanScore> pageInfo = new PageInfo<>(synthesisPlanScores);
        SynthesisPlanScoreDto synthesisPlanScoreDto = new SynthesisPlanScoreDto();
        synthesisPlanScoreDto.setContent(pageInfo);
        //根据userID获取学号和姓名
        UserInfo userInfo = userInfoMapper.selectByPrimaryKey(userId);
        synthesisPlanScoreDto.setStudentId(userInfo.getStudentId());
        //根据caseID获取案例题客户背景
        CaseInfo caseInfo = caseInfoMapper.selectByPrimaryKey(caseId);
        synthesisPlanScoreDto.setCustomerBackdrop(caseInfo.getCustomerBackdrop());
        //获取总成绩
        Double totalScore = synthesisPlanScoreMapper.getTotalScore(userId, caseId);
        synthesisPlanScoreDto.setTotalScore(totalScore);
        return new ResultEntity<>(HttpStatus.OK, "规划报告审核报告展示成功！", synthesisPlanScoreDto);
    }

    //已有客户列表查询
    @AnonymousAccess
    @ApiOperation("已有客户列表查询")
    @PostMapping("selectExistingCustomersList")
    public ResultEntity<PageInfo<SynthesisPlanClient>> selectExistingCustomersList(@RequestParam String userId,
                                                                                   @RequestParam Integer index,
                                                                                   @RequestParam Integer size,
                                                                                   @ApiParam("客户编号/客户姓名/身份证号") @RequestParam(required = false) String param) {
        PageHelper.startPage(index, size);
        SynthesisPlanClientExample example = new SynthesisPlanClientExample();
        SynthesisPlanClientExample.Criteria criteria = example.createCriteria();
        criteria.andUseridEqualTo(userId).andSubmitStatusEqualTo("已审核");
        if (StringUtils.isNotBlank(param)) {
            SynthesisPlanClientExample.Criteria criteria1 = example.createCriteria();
            SynthesisPlanClientExample.Criteria criteria2 = example.createCriteria();
            SynthesisPlanClientExample.Criteria criteria3 = example.createCriteria();
            criteria1.andClientNumberLike("%" + param + "%");
            criteria2.andNameLike("%" + param + "%");
            criteria3.andIdcardLike("%" + param + "%");
            example.or(criteria1);
            example.or(criteria2);
            example.or(criteria3);
        }
        example.setOrderByClause("create_time asc");
        List<SynthesisPlanClient> synthesisPlanClients = synthesisPlanClientMapper.selectByExample(example);
        PageInfo<SynthesisPlanClient> pageInfo = new PageInfo<>(synthesisPlanClients);
        return new ResultEntity<>(HttpStatus.OK, "已有客户列表查询成功！", pageInfo);
    }


    //客户统计
    @AnonymousAccess
    @ApiOperation("客户统计查询")
    @PostMapping("selectUserStatistics")
    public ResultEntity<UserStatisticsDto> selectUserStatistics(@RequestParam String userId) {
        //先获取已有客户总数
        Integer yiYouTotalCount = synthesisPlanClientMapper.selectYiYouTotalCount(userId);
        //获取潜在客户总数
        Integer qianZaiTotalCount = synthesisPlanClientMapper.selectQianZaiTotalCount(userId);

        //获取高净值数据
        UserStatistics userStatistics = userStatisticsMapper.selectByPrimaryKey(userId);
        UserStatisticsDto userStatisticsDto = new UserStatisticsDto();
        userStatisticsDto.setQianZaiTotalCount(qianZaiTotalCount);
        userStatisticsDto.setYiYouTotalCount(yiYouTotalCount);
        userStatisticsDto.setHighNetClientHaveCount(userStatistics==null?0:userStatistics.getHighNetClientHaveCount());
        userStatisticsDto.setHighNetClientLatentCount(userStatistics==null?0:userStatistics.getHighNetClientLatentCount());

        return new ResultEntity<>(HttpStatus.OK, "客户统计查询成功！", userStatisticsDto);
    }

    /*
     * @author xcj
     * @Date 2024/8/15
     */
    @AnonymousAccess
    @ApiOperation("潜在客户列表查询")
    @PostMapping("/getPotentialCustomerList")
    public ResultEntity<PageInfo<SynthesisPlanClient>> getPotentialCustomerList(@RequestParam Integer index,
                                                                                @RequestParam Integer size,
                                                                                @RequestParam(required = false) String keyWord) {
        PageHelper.startPage(index, size);
        List<SynthesisPlanClient> list = synthesisPlanClientMapper.getPotentialCustomerList(keyWord);
        PageInfo pageInfo = new PageInfo(list);
        return new ResultEntity<PageInfo<SynthesisPlanClient>>(pageInfo);
    }

    /*
     * @author xcj
     * @Date 2024/8/15
     */
    @AnonymousAccess
    @ApiOperation("潜在客户单个查询")
    @PostMapping("/getPotentialCustomerById")
    public ResultEntity<SynthesisPlanClient> getPotentialCustomerById(@RequestParam String id) {
        SynthesisPlanClient synthesisPlanClient = synthesisPlanClientMapper.selectByPrimaryKey(id);
        return new ResultEntity<>(synthesisPlanClient);
    }

    /* 编辑建议书
     * @author xcj
     * @Date 2024/8/15
     */
    @AnonymousAccess
    @ApiOperation("编辑潜在客户信息/完善客户信息")
    @PostMapping("/updatePotentialCustomer")
    public ResultEntity<HttpStatus> updatePotentialCustomer(@RequestBody SynthesisPlanClient synthesisPlanClient) {
        if(StringUtils.isBlank(synthesisPlanClient.getProposalNumber())){
            String number = "FPR"+IdUtil.simpleUUID();
            synthesisPlanClient.setProposalNumber(number);
        }
        int i = synthesisPlanClientMapper.updateByPrimaryKeySelective(synthesisPlanClient);
        if (i > 0) {
            return new ResultEntity<>(HttpStatus.OK, "编辑成功！");
        }
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "系统！");
    }


    /*
     * @author xcj
     * @Date 2024/8/15
     */
    @AnonymousAccess
    @ApiOperation("家属信息查询")
    @PostMapping("/getFamilyInfo")
    public ResultEntity<List<ClientFamilyInfo>> getFamilyInfo(@RequestParam String caseId, @RequestParam String userId) {
        ClientFamilyInfoExample example = new ClientFamilyInfoExample();
        example.createCriteria().andCaseidEqualTo(caseId).andUserIdEqualTo(userId);
        example.setOrderByClause("create_time");
        List<ClientFamilyInfo> clientFamilyInfos = clientFamilyInfoMapper.selectByExample(example);
        return new ResultEntity<>(clientFamilyInfos);
    }

    /*
     * @author xcj
     * @Date 2024/8/15
     */
    @AnonymousAccess
    @ApiOperation("编辑或新增家属信息")
    @PostMapping("/updateOrInsertFamilyInfo")
    @Transactional
    public ResultEntity<HttpStatus> updateOrInsertFamilyInfo(@ApiParam("没带ID和创建时间为新增，带了为修改") @RequestBody List<ClientFamilyInfo> list) {
        try {
            for (ClientFamilyInfo clientFamilyInfo : list) {
                //没ID为新增
                if (StringUtils.isBlank(clientFamilyInfo.getClientFamilyInfoId())) {
                    clientFamilyInfo.setClientFamilyInfoId(IdUtil.randomUUID());
                    clientFamilyInfo.setCreateTime(new Date());
                    clientFamilyInfoMapper.insert(clientFamilyInfo);
                }
                //编辑
                else {
                    clientFamilyInfoMapper.updateByPrimaryKeySelective(clientFamilyInfo);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "系统异常，请联系管理员");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "操作成功！");
    }


    @AnonymousAccess
    @ApiOperation("删除家属信息")
    @PostMapping("/deleteFamilyInfo")
    public ResultEntity<HttpStatus> deleteFamilyInfo(@RequestParam String id) {
        int i = clientFamilyInfoMapper.deleteByPrimaryKey(id);
        if (i > 0) {
            return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "系统异常，请联系管理员！");
    }

    @AnonymousAccess
    @ApiOperation("客户背景展示")
    @PostMapping("/getCustomerBackground")
    public ResultEntity<String> getCustomerBackground(@RequestParam String caseId) {
        CaseInfo caseInfo = caseInfoMapper.selectByPrimaryKey(caseId);
        if (caseInfo != null) {
            return new ResultEntity<>(caseInfo.getCustomerBackdrop());
        }
        return null;
    }

    @AnonymousAccess
    @ApiOperation("风险测评结果/建议投资产品分配")
    @PostMapping("/adviseInvestmentProductAllot")
    public ResultEntity<Map<String, String>> adviseInvestmentProductAllot(@ApiParam("风险容忍态度分数") @RequestParam BigDecimal RAIScore,
                                                                          @ApiParam("风险承受能力分数") @RequestParam BigDecimal RCIScore) {
        Map<String, String> map = synthesisPlanService.adviseInvestmentProductAllot(RAIScore, RCIScore);
        return new ResultEntity<Map<String, String>>(map);
    }
}
