package com.sztzjy.money_management.service.impl;

import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.StuClientCaseScoreDto;
import com.sztzjy.money_management.entity.dto.StuSynthesisPlanScoreDto;
import com.sztzjy.money_management.entity.dto.StuTrainingDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.TrainingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class TrainingServiceImpl implements TrainingService {
    @Autowired
    StuTrainingMapper trainingMapper;
    @Autowired
    StudentScoreWeightMapper studentScoreWeightMapper;
    @Autowired
    TrainingReportMapper trainingReportMapper;
    @Autowired
    ClientWeightMapper clientWeightMapper;
    @Autowired
    SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    SynthesisPlanScoreMapper synthesisPlanScoreMapper;
    @Autowired
    TotalWeightMapper totalWeightMapper;

    //获取用户所有内置章节下的分数、时长、权重及得分
    @Override
    public List<StuClientCaseScoreDto> selectStuClientCaseScoreList(String userId, String schoolId) {
        //先根据userId查询学生实训表中的数据
        StuTrainingExample stuTrainingExample = new StuTrainingExample();
        stuTrainingExample.createCriteria().andUserIdEqualTo(userId);
        List<StuTraining> stuTrainingList = trainingMapper.selectByExample(stuTrainingExample);
        if(stuTrainingList.isEmpty()){
            return null;
        }

        //查询全部章节下的学习成绩权重
        Map<String, StudentScoreWeight> studentScoreWeightMap = getStudentScoreWeightMap(schoolId);
        //获取客户案例权重
        Map<String, BigDecimal> clientWeightMap = getClientWeight(schoolId);
        List<StuClientCaseScoreDto> returnList=new ArrayList<>();
        for (int i = 0; i < stuTrainingList.size(); i++) {
            StuClientCaseScoreDto stuClientCaseScoreDto = new StuClientCaseScoreDto();
            StuTraining stuTraining = stuTrainingList.get(i);
            Integer knowledgeStatus = stuTraining.getKnowledgeSummaryCompleteStatus() == null ? 0 : stuTraining.getKnowledgeSummaryCompleteStatus();
            Integer resourceStatus = stuTraining.getResourceLearningCompleteStatus()== null ? 0 : stuTraining.getResourceLearningCompleteStatus();
            Integer learningEvalCompleteStatus = stuTraining.getLearningEvalCompleteStatus()== null ? 0 : stuTraining.getLearningEvalCompleteStatus();
            Integer expStatus = stuTraining.getExpTrainingCompleteStatus()==null?0:stuTraining.getExpTrainingCompleteStatus();
            StudentScoreWeight studentScoreWeight = studentScoreWeightMap.get(stuTraining.getChapterName());
            if(studentScoreWeight==null){
                continue;
            }
            BigDecimal summaryScore = studentScoreWeight.getSummaryOfKnowledgeScore().multiply(BigDecimal.valueOf(knowledgeStatus)).multiply(studentScoreWeight.getSummaryOfKnowledgeWeight());
            BigDecimal resourceScore = studentScoreWeight.getResourceLearningScore().multiply(BigDecimal.valueOf(resourceStatus)).multiply(studentScoreWeight.getResourceLearningWeight());
            BigDecimal learningScore = studentScoreWeight.getLearningAssessmentScore().multiply(BigDecimal.valueOf(learningEvalCompleteStatus)).multiply(studentScoreWeight.getLearningAssessmentWeight());
            BigDecimal experimentalScore = studentScoreWeight.getExperimentalTrainingScore().multiply(BigDecimal.valueOf(expStatus)).multiply(studentScoreWeight.getExperimentalTrainingWeight());
            //设置实验报告情况及得分
            BigDecimal reportScore=BigDecimal.ZERO;
            TrainingReportExample trainingReportExample = new TrainingReportExample();
            trainingReportExample.createCriteria().andChapterIdEqualTo(stuTraining.getChapterName()).andUserIdEqualTo(userId);
            List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
            if(!trainingReports.isEmpty()){
                reportScore=trainingReports.get(0).getTeacherScore().multiply(studentScoreWeight.getReportWeight());
            }
            stuClientCaseScoreDto.setChapterName(stuTraining.getChapterName());
            stuClientCaseScoreDto.setTrainingTotalScore(summaryScore.add(resourceScore).add(learningScore).add(experimentalScore).add(reportScore));
            stuClientCaseScoreDto.setScoreWight(clientWeightMap.get(stuTraining.getChapterName()));
            stuClientCaseScoreDto.setTrainingTime(stuTraining.getTrainingTime());
            stuClientCaseScoreDto.setTrainingScore(stuClientCaseScoreDto.getTrainingTotalScore().multiply(stuClientCaseScoreDto.getScoreWight()));
            returnList.add(stuClientCaseScoreDto);
        }
        return returnList;
    }


    //获取学习成绩权重对象 如果没设置 则用默认方式  map("风险测评" 权重对象)
    @Override
    public Map<String,StudentScoreWeight> getStudentScoreWeightMap(String schoolId){
       Map<String,StudentScoreWeight> map=new HashMap<>();
        List<String> chapterNameList = getChapterNameList();
        StudentScoreWeight studentScoreWeight = new StudentScoreWeight();
        //计分规则
        studentScoreWeight.setSummaryOfKnowledgeScore(BigDecimal.valueOf(20));
        studentScoreWeight.setResourceLearningScore(BigDecimal.valueOf(2));
        studentScoreWeight.setLearningAssessmentScore(BigDecimal.valueOf(10));
        studentScoreWeight.setExperimentalTrainingScore(BigDecimal.valueOf(20));

        //权重
        studentScoreWeight.setSummaryOfKnowledgeWeight(BigDecimal.valueOf(0.2));
        studentScoreWeight.setResourceLearningWeight(BigDecimal.valueOf(0.2));
        studentScoreWeight.setLearningAssessmentWeight(BigDecimal.valueOf(0.2));
        studentScoreWeight.setExperimentalTrainingWeight(BigDecimal.valueOf(0.2));
        studentScoreWeight.setReportWeight(BigDecimal.valueOf(0.2));
        for (int i = 0; i < chapterNameList.size(); i++) {  //先将所有内置章节设置成默认章节
            map.put(chapterNameList.get(i),studentScoreWeight);
        }

        //获取老师设置的权重 如果没有设置 则将map返回  如果设置了部分 则将map中设置过的章节替换成老师设置的章节
        StudentScoreWeightExample studentScoreWeightExample = new StudentScoreWeightExample();
        studentScoreWeightExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<StudentScoreWeight> studentScoreWeightList = studentScoreWeightMapper.selectByExample(studentScoreWeightExample);
        if(studentScoreWeightList.isEmpty()){
            return map;
        }else {
            for (int i = 0; i < studentScoreWeightList.size(); i++) {
                StudentScoreWeight teaWeight = studentScoreWeightList.get(i);
                map.put(studentScoreWeight.getChapterName(),teaWeight);
            }
            return map;
        }
    }

    //获取客户案例权重对象 map("风险测评" 权重数值)
    public Map<String,BigDecimal> getClientWeight(String schoolId){
        Map<String,BigDecimal> map=new HashMap<>();
        ClientWeight clientWeight = clientWeightMapper.selectByPrimaryKey(schoolId);
        if(clientWeight==null){  //设置默认权重
            map.put("风险测评",BigDecimal.valueOf(0.05));
            map.put("财务分析",BigDecimal.valueOf(0.20));
            map.put("现金规划",BigDecimal.valueOf(0.05));
            map.put("教育规划",BigDecimal.valueOf(0.1));
            map.put("消费规划",BigDecimal.valueOf(0.1));
            map.put("创业规划",BigDecimal.valueOf(0.05));
            map.put("退休规划",BigDecimal.valueOf(0.1));
            map.put("保险规划",BigDecimal.valueOf(0.15));
            map.put("投资规划",BigDecimal.valueOf(0.05));
            map.put("税务筹划",BigDecimal.valueOf(0.05));
            map.put("财产分配",BigDecimal.valueOf(0.05));
            map.put("财产传承",BigDecimal.valueOf(0.05));
            return map;
        }else {
            map.put("风险测评",clientWeight.getFxcp());
            map.put("财务分析",clientWeight.getCwfx());
            map.put("现金规划",clientWeight.getXjgh());
            map.put("教育规划",clientWeight.getJygh());
            map.put("消费规划",clientWeight.getXfgh());
            map.put("创业规划",clientWeight.getCygh());
            map.put("退休规划",clientWeight.getTxgh());
            map.put("保险规划",clientWeight.getBxgh());
            map.put("投资规划",clientWeight.getTzgh());
            map.put("税务筹划",clientWeight.getSwch());
            map.put("财产分配",clientWeight.getCcfp());
            map.put("财产传承",clientWeight.getCccc());
            return map;
        }
    }

    //设置所有的章节名称
    @Override
    public List<String> getChapterNameList(){
        List<String> list=new ArrayList<>();
        list.add("风险测评");
        list.add("财务分析");
        list.add("现金规划");
        list.add("教育规划");
        list.add("消费规划");
        list.add("创业规划");
        list.add("退休规划");
        list.add("保险规划");
        list.add("投资规划");
        list.add("税务筹划");
        list.add("财产分配");
        list.add("财产传承");
        return list;
    }



    //学生端 综合规划成绩查询
    // 学习项目（案例名称）-》综合规划表中查询用户姓名
    // 实训时长 -》综合规划表中查询
    // 实训总分 -》综合规划成绩表中查询
    // 成绩权重 -》totalweight表中查询
    // 完成情况-》设置为已完成
    // 实训得分 -》总分*权重

    @Override
    public List<StuSynthesisPlanScoreDto> selectStuSynthesisPlanScoreList(String userId, String schoolId) {
        SynthesisPlanClientExample clientExample = new SynthesisPlanClientExample();
        clientExample.createCriteria().andUseridEqualTo(userId).andSubmitStatusEqualTo("已审核");
        //获取该用户所有审核通过的综合规划数据
        List<SynthesisPlanClient> synthesisPlanClientList = synthesisPlanClientMapper.selectByExample(clientExample);
        if(synthesisPlanClientList.isEmpty()){
            return null;
        }
        //获取该学校的成绩总览权重数据
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        BigDecimal zhghWeight= BigDecimal.valueOf(0.4);
        if(totalWeight!=null){
            zhghWeight = totalWeight.getZhgh();
        }
        List<StuSynthesisPlanScoreDto> returnList=new ArrayList<>();
        for (int i = 0; i < synthesisPlanClientList.size(); i++) {
            StuSynthesisPlanScoreDto returnDto = new StuSynthesisPlanScoreDto();
            SynthesisPlanClient client = synthesisPlanClientList.get(i);
            returnDto.setName(client.getName());
            returnDto.setUseTime(client.getUseTime());
            //获取实训总分
            BigDecimal trainingTotalScore=synthesisPlanScoreMapper.selectTotalScoreByUserIdAndCaseId(userId,client.getCaseid());
            returnDto.setTrainingTotalScore(trainingTotalScore);
            returnDto.setScoreWight(zhghWeight);
            returnDto.setFinishStatus("已完成");
            returnDto.setTrainingScore(trainingTotalScore.multiply(zhghWeight));
            returnList.add(returnDto);
        }
        return returnList;
    }
}
